@@ -11,39 +11,24 @@ BINDING_MODES = {'ro': 'ro', 'w': 'rw', 'rw': 'rw'}
 
 class Singularity(Backend):
     def __init__(self, **kwargs):
+        super(Singularity, self).__init__(**kwargs)
         self.platform = Platform_Meta('Singularity', 'Ⓢ')
-        image = kwargs.get("image")
-        tag = kwargs.get("tag")
-        pwd = os.getcwd()
-        if kwargs.get("working_dir") is not None:
-            pwd = kwargs["working_dir"]
-            os.chdir(pwd)
         self._print_loading_with_symbol(self.platform.name)
-        if image and isinstance(image, str) and os.path.exists(image):
-            self.image = image
-        elif tag and isinstance(tag, str):  # pragma: no cover
-            self.image = Client.pull(
-                f"docker://{image}:{tag}",
-                pull_folder=pwd
-            )
-        else:  # pragma: no cover
-            try:
-                self.image = Client.pull(
-                    "shub://FCP-INDI/C-PAC",
-                    pull_folder=pwd
-                )
-            except Exception:
-                try:
-                    self.image = Client.pull(
-                        f"docker://fcpindi/c-pac:latest",
-                        pull_folder=pwd
-                    )
-                except Exception:
-                    raise OSError("Could not connect to Singularity")
+        self.pull(**kwargs, force=False)
         self.volumes = {}
         self.options = list(chain.from_iterable(kwargs[
             "container_options"
         ])) if bool(kwargs.get("container_options")) else []
+        if isinstance(self.pipeline_config, str):
+            self.config = Client.execute(
+                image=self.image,
+                command=f'cat {self.pipeline_config}',
+                return_result=False
+            )
+        else:
+            self.config = self.pipeline_config
+        kwargs = self.collect_config_bindings(self.config, **kwargs)
+        del self.config
         self._set_bindings(**kwargs)
 
     def _bindings_as_option(self):
@@ -60,7 +45,44 @@ class Singularity(Backend):
             ] for local in self.volumes])))]
         )
 
-    def _try_to_stream(self, args, stream_command='run'):
+    def pull(self, force=False, **kwargs):
+        image = kwargs['image'] if kwargs.get(
+            'image'
+        ) is not None else 'fcpindi/c-pac'
+        tag = kwargs.get("tag")
+        pwd = os.getcwd()
+        if kwargs.get("working_dir") is not None:
+            pwd = kwargs["working_dir"]
+            os.chdir(pwd)
+        if (
+            not force and
+            image and isinstance(image, str) and os.path.exists(image)
+        ):
+            self.image = image
+        elif tag and isinstance(tag, str):  # pragma: no cover
+            self.image = Client.pull(
+                f"docker://{image}:{tag}",
+                force=force,
+                pull_folder=pwd
+            )
+        else:  # pragma: no cover
+            try:
+                self.image = Client.pull(
+                    "shub://FCP-INDI/C-PAC",
+                    force=force,
+                    pull_folder=pwd
+                )
+            except Exception:
+                try:
+                    self.image = Client.pull(
+                        "docker://fcpindi/c-pac:latest",
+                        force=force,
+                        pull_folder=pwd
+                    )
+                except Exception:
+                    raise OSError("Could not connect to Singularity")
+
+    def _try_to_stream(self, args, stream_command='run', **kwargs):
         self._bindings_as_option()
         try:
             if stream_command == 'run':
@@ -84,9 +106,11 @@ class Singularity(Backend):
         except CalledProcessError:  # pragma: no cover
             return
 
-    def _read_crash(self, read_crash_command):
+    def _read_crash(self, read_crash_command, **kwargs):
         return self._try_to_stream(
-            args={'command': read_crash_command}, stream_command='execute'
+            args={'command': read_crash_command},
+            stream_command='execute',
+            **kwargs
         )
 
     def run(self, flags=[], **kwargs):
